Function.RegisterNamespace("xUnit.js.Console.Output.OutputStrategy");

xUnit.js.Console.Output.OutputStrategy.Json=function(){
    // Private Members
    var _output;

    // IOutputStrategy Members
    this.OutputLevel="";

    this.Prologue=function(){
        _output={
            errors:[],
            files:[],
            facts:[]
        };
    };

    this.Epilogue=function(){
        System.Environment.Write(new System.Script.ObjectSerializer().Serialize(_output));
    };

    this.BeginFileLoad=function(){};

    this.FileLoadSuccess=function(file,duration){};

    this.CompleteFileLoad=function(files,duration){
        Array.ForEach(files,listFile);
        _output.files.duration=duration/1000;
    };

    this.BeginRun=function(){
        _output.facts.length=0;
        _output.run={
            count:0,
            errors:0,
            failures:0,
            skipped:0,
            duration:0,
            timestamp:new Date()
        };
    };

    this.CompleteRun=function(successes,failures,errors,skipped,duration){
        var count=successes.length+failures.length+errors.length+skipped.length;
        _output.run.count=count;
        _output.run.failures=failures.length;
        _output.run.errors=errors.length;
        _output.run.skipped=skipped.length;
        _output.run.duration=duration/1000;
        if(errors.length){
            _output.run.errors=[];
            Array.ForEach(errors,listError);
        }
        if(failures.length){
            _output.run.failures=[];
            Array.ForEach(failures,listFailure);
        }
        if(skipped.length){
            _output.run.skipped=[];
            Array.ForEach(skipped,listSkipped);
        }
    };
    
    this.BeginComponent=function(component){
        // no-op
    };

    this.CompleteComponent=function(component,duration){
        if(!Object.IsType(xUnit.js.Model.Fact,component))return;
        _output.facts.push({
            path:getPath(component),
            result:getResult(component.State.Result),
            message:component.State.Message||'',
            duration:duration/1000
        });
    };

    this.Enumerate=function(component){
        _output.facts.push({
            path:getPath(component)
        });
    };

    this.Error=function(error){
        var output={message:error+''};
        if(error&&isVerbose()){
            if(error.lineNumber)output.lineNumber=error.lineNumber;
            if(error.number)output.number=error.number;
            if(error.toSource)output.source=error.toSource();
            if(error.stackTrace||error.stack||error.getStack)output.stack=error.stackTrace||error.stack||error.getStack();
        }
        _output.errors.push(output);
    }
    
    // IStrategySpecification Members
    this.IsSatisfiedBy=function(candidate){
        return String.Equals(xUnit.js.Console.Output.OutputFormatter.OutputTypes.json,candidate);
    };
    
    // Private Methods
    function getPath(fact){
        return fact.GetPath().split('.').slice(1).join('.');
    }
    
    function getResult(result){
        switch(result){
            case xUnit.js.Model.Result.Error:
                return "Error";
            case xUnit.js.Model.Result.Failure:
                return "Failure";
            case xUnit.js.Model.Result.Skipped:
                return "Skipped";
            case xUnit.js.Model.Result.Success:
                return "Success";
        }
        return '';
    }

    function isVerbose(){
        return this.OutputLevel==xUnit.js.Console.Output.OutputFormatter.OutputLevels.verbose;
    }
    isVerbose=Function.GetDelegate(isVerbose,this);
    
    function listError(run,context){
        _output.run.errors.push({
            "file":run.Component.File,
            index:run.State.Index,
            path:getPath(run.Component),
            type:run.State.Message&&run.State.Message.name||"Exception",
            message:run.State.Message||"[no message]"
        });
    }

    function listFile(file,context){
        _output.files.push({
            path:file
        });
    }

    function listFailure(run,context){
        _output.run.failures.push({
            "file":run.Component.File,
            index:run.State.Index,
            path:getPath(run.Component),
            type:run.State.Message&&run.State.Message.name||"Exception",
            message:run.State.Message||"[no message]"
        });
    }

    function listSkipped(run,context){
        _output.run.failures.push({
            path:getPath(run.Component),
            message:run.State.Message||"[no message]"
        });
    }    
};

xUnit.js.Console.Output.OutputStrategy.Json.Implement(xUnit.js.Console.Output.IOutputStrategy,'xUnit.js.Console.Output.OutputStrategy.Json');
xUnit.js.Console.Output.OutputStrategy.Json.Implement(System.Script.Strategy.IStrategySpecification,'xUnit.js.Console.Output.OutputStrategy.Json');

xUnit.js.Console.Output.OutputFormatter.Strategies.Add(xUnit.js.Console.Output.OutputStrategy.Json);
